@@ -6,7 +6,7 @@ and name(s), a comment, or a blank line/line separator.
 
 Hosts:
 A representation of a hosts file, e.g. /etc/hosts and
-c:\\\windows\\\system32\\\drivers\\\etc\\\hosts for a linux or MS windows
+c:\\\\windows\\\\system32\\\\drivers\\\\etc\\\\hosts for a linux or MS windows
 based machine respectively. Each entry being represented as an instance
 of the HostsEntry class.
 """
@@ -193,7 +193,7 @@ class Hosts(object):
             with open(output_file_path, 'w') as hosts_file:
                 for written_count, line in enumerate(self.entries):
                     if line.entry_type == 'comment':
-                        hosts_file.write(line.comment)
+                        hosts_file.write(line.comment + "\n")
                         comments_written += 1
                     if line.entry_type == 'blank':
                         hosts_file.write("\n")
@@ -230,12 +230,13 @@ class Hosts(object):
         response = urlopen(url)
         return response.read()
 
-    def exists(self, address=None, names=None):
+    def exists(self, address=None, names=None, comment=None):
         """
-        Determine if the supplied address and/or names exist in a HostsEntry within Hosts
+        Determine if the supplied address and/or names, or comment, exists in a HostsEntry within Hosts
         :param address: An ipv4 or ipv6 address to search for
         :param names: A list of names to search for
-        :return: True if a supplied address or name is found. Otherwise, False.
+        :param comment: A comment to search for
+        :return: True if a supplied address, name, or comment is found. Otherwise, False.
         """
         for entry in self.entries:
             if entry.entry_type in ('ipv4', 'ipv6'):
@@ -245,6 +246,8 @@ class Hosts(object):
                     for name in names:
                         if name in entry.names:
                             return True
+            elif entry.entry_type == 'comment' and entry.comment == comment:
+                return True
         return False
 
     def remove_all_matching(self, address=None, name=None):
@@ -266,6 +269,28 @@ class Hosts(object):
                 raise ValueError('No address or name was specified for removal.')
             self.entries = list(filter(func, self.entries))
 
+    def find_all_matching(self, address=None, name=None):
+        """
+        Return all HostsEntry instances from the Hosts object
+        where the supplied ip address or name matches
+        :param address: An ipv4 or ipv6 address
+        :param name: A host name
+        :return: HostEntry instances
+        """
+        results = []
+        if self.entries:
+            for entry in self.entries:
+                if not entry.is_real_entry():
+                    continue
+                if address and name:
+                    if address == entry.address and name in entry.names:
+                        results.append(entry)
+                elif address and address == entry.address:
+                    results.append(entry)
+                elif name in entry.names:
+                    results.append(entry)
+        return results
+
     def import_url(self, url=None, force=None):
         """
         Read a list of host entries from a URL, convert them into instances of HostsEntry and
@@ -331,16 +356,18 @@ class Hosts(object):
             return {'result': 'failed',
                     'message': 'Cannot read: file {0}.'.format(import_file_path)}
 
-    def add(self, entries=None, force=False, allow_address_duplication=False):
+    def add(self, entries=None, force=False, allow_address_duplication=False, merge_names=False):
         """
         Add instances of HostsEntry to the instance of Hosts.
         :param entries: A list of instances of HostsEntry
         :param force: Remove matching before adding
         :param allow_address_duplication: Allow using multiple entries for same address
+        :param merge_names: Merge names where address already exists
         :return: The counts of successes and failures
         """
         ipv4_count = 0
         ipv6_count = 0
+        comment_count = 0
         invalid_count = 0
         duplicate_count = 0
         replaced_count = 0
@@ -352,8 +379,13 @@ class Hosts(object):
                 existing_names.extend(item.names)
         existing_names = dedupe_list(existing_names)
         for entry in entries:
-            # Allow duplicates entries for addresses used for adblocking
-            if (entry.address in ('0.0.0.0', '127.0.0.1') or allow_address_duplication):
+            if entry.entry_type == 'comment':
+                entry.comment = entry.comment.strip()
+                if entry.comment[0] != "#":
+                    entry.comment = "# " + entry.comment
+                import_entries.append(entry)
+            elif entry.address in ('0.0.0.0', '127.0.0.1') or allow_address_duplication:
+                # Allow duplicates entries for addresses used for adblocking
                 if set(entry.names).intersection(existing_names):
                     if force:
                         for name in entry.names:
@@ -364,8 +396,22 @@ class Hosts(object):
                 else:
                     import_entries.append(entry)
             elif entry.address in existing_addresses:
-                if not force:
+                if not any((force, merge_names)):
                     duplicate_count += 1
+                elif merge_names:
+                    # get the last entry with matching address
+                    entry_names = list()
+                    for existing_entry in self.entries:
+                        if entry.address == existing_entry.address:
+                            entry_names = existing_entry.names
+                            break
+                    # merge names with that entry
+                    merged_names = list(set(entry.names + entry_names))
+                    # remove all matching
+                    self.remove_all_matching(address=entry.address)
+                    # append merged entry
+                    entry.names = merged_names
+                    import_entries.append(entry)
                 elif force:
                     self.remove_all_matching(address=entry.address)
                     replaced_count += 1
@@ -382,13 +428,17 @@ class Hosts(object):
                 import_entries.append(entry)
 
         for item in import_entries:
-            if item.entry_type == 'ipv4':
+            if item.entry_type == 'comment':
+                comment_count += 1
+                self.entries.append(item)
+            elif item.entry_type == 'ipv4':
                 ipv4_count += 1
                 self.entries.append(item)
             elif item.entry_type == 'ipv6':
                 ipv6_count += 1
                 self.entries.append(item)
-        return {'ipv4_count': ipv4_count,
+        return {'comment_count': comment_count,
+                'ipv4_count': ipv4_count,
                 'ipv6_count': ipv6_count,
                 'invalid_count': invalid_count,
                 'duplicate_count': duplicate_count,
@@ -406,6 +456,8 @@ class Hosts(object):
                 for hosts_entry in hosts_entries:
                     entry_type = HostsEntry.get_entry_type(hosts_entry)
                     if entry_type == "comment":
+                        hosts_entry = hosts_entry.replace("\r", "")
+                        hosts_entry = hosts_entry.replace("\n", "")
                         self.entries.append(HostsEntry(entry_type="comment",
                                                        comment=hosts_entry))
                     elif entry_type == "blank":
